const editor = document.querySelector('.article-editor');
const options = {
  placeholder: '请输入正文',
  scrollingContainer: document.documentElement
};
const quill = new Quill(editor, options);

/**
 * 注册编辑器分割线
 */
const BlockEmbed = Quill.import('blots/block/embed');
const Link = Quill.import('formats/link');

class DividerBlot extends BlockEmbed { }

// 分割线
DividerBlot.blotName = 'divider';
DividerBlot.tagName = 'hr';

Quill.register(DividerBlot);

// title、summary相关事件
const titleTextareaContent = document.querySelector('.title-textarea-content');
const titleTextarea = document.querySelector('.title-textarea');
const summaryTextareaContent = document.querySelector('.summary-textarea-content');
const summaryTextarea = document.querySelector('.summary-textarea');
const titleError = document.querySelector('.title-error-message');
const summaryMaxlength = document.querySelector('.summary-maxlength span');
const editorMaxlength = document.querySelector('.editor-maxlength span');
let isComposing = false;
let isEditorFocus = false;
let keyboardHeight = 0;

// 监听标题输入框输入事件
titleTextarea.addEventListener('input', (event) => {
  let value = event.target.value;
  value = value.substring(0, 30);

  if (value.indexOf('\n') !== -1) {
    value = value.replace(/\n/g, '');
    summaryTextarea.focus();
    summaryTextarea.setSelectionRange(summaryTextarea.value.length, summaryTextarea.value.length);
  }

  // 清空标题错误提示
  titleError.innerHTML = '';
  titleTextarea.value = value;
  titleTextareaContent.innerHTML = value;
  callWebviewMethod('jkttTitleChange', JSON.stringify({
    text: value,
    length: value.length
  }));
})

titleTextarea.addEventListener('blur', () => {
  const length = titleTextarea.value.length
  if (length < 5) {
    titleError.innerHTML = '标题字数不可少于5个字';
  }
})

// 监听摘要输入框输入事件
summaryTextarea.addEventListener('input', (event) => {
  let value = event.target.value;
  value = value.substring(0, 200);

  if (value.indexOf('\n') !== -1) {
    value = value.replace(/\n/g, '');
    quill.focus();
    quill.setSelection(0);
  }

  summaryTextarea.value = value;
  summaryTextareaContent.innerHTML = value;
  summaryMaxlength.innerHTML = summaryTextarea.value.length;
  callWebviewMethod('jkttSummaryChange', JSON.stringify({
    text: value,
    length: value.length
  }));
})

// 编辑器change事件
quill.on('text-change', handleEditorChange);

function handleEditorChange() {
  if (isComposing) return;
  let html = editor.children[0].innerHTML;
  const text = quill.getText().trim().replace(/\n/g, '');

  if (html === '<p><br></p>') html = '';

  const dataMap = {
    text,
    html,
    history: quill.history.stack,
    format: quill.getFormat(),
    length: text.length
  }

  editorMaxlength.innerHTML = text.length;
  callWebviewMethod('jkttContentChange', JSON.stringify(dataMap));
  setTimeout(() => {
    handleCursorScroll();
  })
}

function handleCursorScroll() {
  if (window.platform.toLowerCase() === 'ios') {
    if (keyboardHeight === 0) return
    const scrollTop = document.documentElement.scrollTop || document.body.scrollTop;
    const offsetTop = document.querySelector('.ql-container').offsetTop;
    const selectionTop = quill.getBounds(quill.getSelection(true).index).top;
    const cursorTop = selectionTop + offsetTop - scrollTop;

    if ((keyboardHeight - cursorTop) < 10) {
      if (navigator.userAgent.includes('12_0')) {
        document.body.scrollTop = scrollTop + (cursorTop - keyboardHeight) + 50;
      } else {
        document.documentElement.scrollTop = scrollTop + (cursorTop - keyboardHeight) + 50;
      }
    }
  }
}

// 编辑器光标改变事件
quill.on('selection-change', (range, oldRange, source) => {
  if (range) {
    callWebviewMethod('jkttEditorFocus', quill.getFormat())
    isEditorFocus = true;
    setTimeout(() => {
      handleCursorScroll();
    })
  } else {
    callWebviewMethod('jkttEditorBlur')
    isEditorFocus = false;
  }
});

quill.root.addEventListener('compositionstart', () => {
  isComposing = true;
})

quill.root.addEventListener('compositionupdate', () => {
  isComposing = true;
})

quill.root.addEventListener('compositionend', () => {
  if (isComposing) {
    isComposing = false;
    handleEditorChange();
  }
})

// 编辑器格式化内容
window.formatText = (name, value) => {
  quill.format(name, value);
}

// 插入图片
window.insertImage = (urls) => {
  const images = urls.split(',');

  images.map((item) => {
    quill.insertEmbed(quill.getSelection(true).index, 'image', item);
    quill.insertText(quill.getSelection(true).index, '\n');
    quill.setSelection(quill.getSelection(true).index + 1);
  })
  quill.insertText(quill.getSelection(true).index, '\n');
}

// 插入分割线
window.insertDivider = () => {
  const length = quill.getSelection(true).index;
  quill.insertEmbed(length, 'divider', true);
  quill.setSelection(length + 1);
}

// 撤销/恢复
window.reOrUndo = (type) => {
  if (type === '1') {
    quill.history.redo();
  } else {
    quill.history.undo();
  }
}

// 插入emoji
window.insertEmoji = (unicode) => {
  const length = quill.getSelection(true).index;
  quill.insertText(length, unicode);
}

// 插入话题
window.insertTopic = (topic) => {
  const length = quill.getSelection(true).index;
  // 插入话题
  quill.insertText(length, topic, 'link', topic);
  // quill.insertText(length, '123123');
  // quill.setSelection(length + topic.length);
  // quill.focus();
}

// 提取话题和文章图片列表
window.getTopicAndCoverList = () => {
  const html = quill.container.firstChild.innerHTML
  const matchTopic = html.match(/(#(?!\s*#)[^#]+#)/g)
  const topic = matchTopic ? JSON.stringify(matchTopic.map((item) => item.replace(/#/g, ''))) : ''
  const imgDomList = [...document.querySelectorAll('.ql-editor img')]

  callWebviewMethod('jkttTopicListChange', topic)
  callWebviewMethod('jkttAutoCoverListChange', JSON.stringify(imgDomList.map((item) => item.src)));
}

// 删除文本
window.deleteText = () => {
  const length = quill.getSelection(true).index;
  quill.deleteText(length - 1, 1);
}

// 设置平台
window.setPlatform = function (platform) {
  window.platform = platform;
}

// 设置数据
window.setData = function (title, summary, textContent) {
  titleTextarea.value = title;
  titleTextareaContent.innerHTML = title;
  summaryTextarea.value = summary;
  summaryTextareaContent.innerHTML = summary;
  summaryMaxlength.innerHTML = summary.length;
  quill.pasteHTML(textContent);
  callWebviewMethod('jkttTitleChange', JSON.stringify({
    text: title,
    length: title.length
  }));
  callWebviewMethod('jkttSummaryChange', JSON.stringify({
    text: summary,
    length: summary.length
  }));
}

// 设置数据
window.setSelection = () => {
  quill.focus()
  // quill.getSelection(true).index
}

window.setKeyboardHeight = (height) => {
  keyboardHeight = height;
}

window.scrollWindow = (toolHeight) => {
  // summaryTextarea.value = height;
  // const scrollTop = document.documentElement.scrollTop;
  // const editorTop = document.querySelector('.article-editor').offsetTop;
  // if (isEditorFocus && scrollTop > 0) {
  //   // summaryTextarea.value = `聚焦了,距离是${scrollTop}`;
  //   const clientHeight = document.documentElement.clientHeight;
  //   const maxVisibleHeight = clientHeight - toolHeight;
  //   summaryTextarea.value = `${maxVisibleHeight}`;
  // } else {
  //   summaryTextarea.value = '没有';
  // }
}

// const btn = document.querySelector('.setting-btn');
// const btn1 = document.querySelector('.setting-btn1');
//
// btn.addEventListener('click', () => {
//   alert(JSON.stringify(quill.getBounds(quill.getSelection(true).index)))
// })
//
// btn1.addEventListener('click', () => {
//   document.documentElement.scrollTop = 200;
//   console.log(quill.getSelection(true).index);
// })

function callWebviewMethod(name, value) {
  if (window.platform) {
    if (window.platform.toLowerCase() === 'ios') {
      window.webkit.messageHandlers[name].postMessage(value);
    } else {
      jktt[name](value ? value : '');
    }
  }
}
